"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
async function setPublicContentTypes({ strapi, actions = {}, plugins = {}, maxParallelOperations, verbose = false, }) {
    function log(...args) {
        if (verbose) {
            strapi.log.info(...args);
        }
    }
    function warn(...args) {
        strapi.log.warn(...args);
    }
    if (typeof maxParallelOperations === "number") {
        warn(`"maxParallelOperations" configuration option is deprecated. It no longer has any effect.`);
    }
    if ((0, helpers_1.isEmpty)({ ...actions, ...plugins })) {
        warn(`No actions or plugins found in public-permissions plugin config.`);
        return;
    }
    log(`---------------------------------------`);
    log(`Setting permissions to public...`);
    const configuredActions = Object.entries((0, helpers_1.replaceObjectKeyWithApiId)((0, helpers_1.replaceWildcardWithModelNames)(strapi, actions)));
    const configuredPlugins = Object.entries((0, helpers_1.replaceObjectKeyWithPluginId)(plugins));
    const { toDelete, toInsert } = (0, helpers_1.createDbOperationsLists)([
        ...configuredActions,
        ...configuredPlugins,
    ]);
    await strapi.db.connection.transaction(async function (trx) {
        var _a, _b;
        const publicRole = await helpers_1.db.getPublicRole(trx);
        const publicPermissions = await helpers_1.db.getPublicPermissions(trx);
        const existingPermissions = [];
        const permissionsToDelete = [];
        const permissionsToInsert = [];
        for (const permission of publicPermissions) {
            if (!permission.action) {
                continue;
            }
            const action = permission.action;
            const model = (_b = (_a = action.match(/([\w-:.]+)\..+$/)) === null || _a === void 0 ? void 0 : _a[1]) !== null && _b !== void 0 ? _b : "";
            const toInsertIncludesAction = toInsert.includes(action);
            const toDeleteIncludesModel = toDelete.includes(model);
            if (toInsertIncludesAction) {
                existingPermissions.push(permission);
            }
            if (!toInsertIncludesAction && toDeleteIncludesModel) {
                permissionsToDelete.push(permission);
            }
        }
        for (const action of toInsert) {
            if (!existingPermissions.find((p) => p.action === action)) {
                permissionsToInsert.push(action);
            }
        }
        await helpers_1.db.deletePermissions(trx, permissionsToDelete);
        log(`Deleted  ${permissionsToDelete.length} old permissions.`);
        const insertedPermissionIds = await helpers_1.db.insertPermissions(trx, permissionsToInsert);
        log(`Inserted ${insertedPermissionIds.length} new permissions.`);
        const permissionIdsThatNeedLinks = [
            ...insertedPermissionIds,
            ...existingPermissions.map(({ id }) => id),
        ];
        const existingLinks = await helpers_1.db.getPermissionsLinksByPermissionIds(trx, permissionIdsThatNeedLinks);
        const linksToInsert = permissionIdsThatNeedLinks.filter((id) => !existingLinks.find((l) => l.permission_id === id));
        await helpers_1.db.insertPermissionLinks(trx, linksToInsert, publicRole.id);
        log(`Finished setting permissions to public!`);
        log(`---------------------------------------`);
    });
}
exports.default = ({ strapi }) => {
    const plugin = strapi.plugin("public-permissions");
    setPublicContentTypes({
        strapi,
        actions: plugin.config("actions"),
        plugins: plugin.config("plugins"),
        maxParallelOperations: plugin.config("maxParallelOperations"),
        verbose: plugin.config("verbose"),
    });
};
